import os
import cv2
from . import find_tools as ft


def find_right_lung_lbp(image):
    right_lung = cv2.CascadeClassifier(os.path.dirname(__file__) + os.sep + "right_lung_lbp.xml")
    found = right_lung.detectMultiScale(image, 1.8, 5)
    right_lung_rectangle = ft.find_max_rectangle(found)

    return right_lung_rectangle


def find_left_lung_lbp(image):
    left_lung = cv2.CascadeClassifier(os.path.dirname(__file__) + os.sep + "left_lung_lbp.xml")
    found = left_lung.detectMultiScale(image, 1.8, 5)
    left_lung_rectangle = ft.find_max_rectangle(found)

    return left_lung_rectangle



from . import haar_finder
from . import hog_finder
from . import lbp_finder


def find_max_rectangle(rectangles):
    max_rectangle = None
    max_area = 0

    for rectangle in rectangles:
        x, y, width, height = rectangle
        area = width * height

        if area > max_area:
            max_area = area
            max_rectangle = rectangle

    return max_rectangle


def get_lungs(image, padding=15):
    right_lung = hog_finder.find_right_lung_hog(image)
    left_lung = hog_finder.find_left_lung_hog(image)

    if right_lung is not None and left_lung is not None:
        x_right, y_right, width_right, height_right = right_lung
        x_left, y_left, width_left, height_left = left_lung

        if abs(x_right - x_left) < min(width_right, width_left):
            right_lung = lbp_finder.find_right_lung_lbp(image)
            left_lung = lbp_finder.find_left_lung_lbp(image)

    if right_lung is None:
        right_lung = haar_finder.find_right_lung_haar(image)

    if left_lung is None:
        left_lung = haar_finder.find_left_lung_haar(image)

    if right_lung is None:
        right_lung = lbp_finder.find_right_lung_lbp(image)

    if left_lung is None:
        left_lung = lbp_finder.find_left_lung_lbp(image)

    if right_lung is None and left_lung is None:
        return None
    elif right_lung is None:
        x, y, width, height = left_lung
        spine = width / 5
        right_lung = int(x - width - spine), y, width, height
    elif left_lung is None:
        x, y, width, height = right_lung
        spine = width / 5
        left_lung = int(x + width + spine), y, width, height

    x_right, y_right, _, height_right = right_lung
    x_left, y_left, width_left, height_left = left_lung
    x_right -= padding
    y_right -= padding
    height_right += padding * 2
    y_left -= padding
    width_left += padding
    height_left += padding * 2

    if x_right < 0:
        x_right = 0

    if y_right < 0:
        y_right = 0

    if x_left < 0:
        x_left = 0

    if y_left < 0:
        y_left = 0

    if y_right + height_right > image.shape[0]:
        height_right = image.shape[0] - y_right

    if x_left + width_left > image.shape[1]:
        width_left = image.shape[1] - x_left

    if y_left + height_left > image.shape[0]:
        height_left = image.shape[0] - y_left

    top_y = min(y_right, y_left)
    bottom_y = max(y_right + height_right, y_left + height_left)

    return image[top_y:bottom_y, x_right:x_left + width_left]



from setuptools import setup, find_packages

setup(
    name="lungs_finder",
    version="1.0.0",
    description="Library that helps you to find lungs on chest X-ray (CXR) images for further processing",
    author="Maksym Kholiavchenko",
    author_email="dirtmaxim@gmail.com",
    url="https://github.com/dirtmaxim/lungs-finder",
    download_url="https://github.com/dirtmaxim/lungs-finder/archive/1.0.0.tar.gz",
    license="Apache-2.0",
    packages=find_packages(),
    package_data={"lungs_finder": ["*.xml", "*.np"]}
)



import os
import numpy as np
import cv2
from . import find_tools as ft

# noinspection PyArgumentList
hog = cv2.HOGDescriptor((32, 64), (16, 16), (8, 8), (8, 8), 9, 1, 4, 0, 0.2, 0, 64)


def find_right_lung_hog(image):
    hog.setSVMDetector(
        np.loadtxt(os.path.dirname(__file__) + os.sep + "right_lung_hog.np", dtype=np.float32))
    found, w = hog.detectMultiScale(image)
    right_lung_rectangle = ft.find_max_rectangle(found)

    return right_lung_rectangle


def find_left_lung_hog(image):
    hog.setSVMDetector(np.loadtxt(os.path.dirname(__file__) + os.sep + "left_lung_hog.np", dtype=np.float32))
    found, w = hog.detectMultiScale(image)
    left_lung_rectangle = ft.find_max_rectangle(found)

    return left_lung_rectangle
